const path = require('path');
const fs = require('fs');
const SVGDoc = require('svg-sprite-loader/lib/svg-document');

const SALESFORCE_UX_ICONS_ROOT = path.resolve(__dirname, '../../@salesforce-ux/design-system/assets/icons');
const TEMP_FOLDER = path.resolve(__dirname, '../.temp');
const categories = ['action', 'custom', 'doctype', 'standard', 'utility'];

const ensureSvg = function(category, name) {
  const promise = new Promise(function(resolve, reject) {
    const tempFile = path.join(TEMP_FOLDER, category, `${name}.js`);
    if (fs.existsSync(tempFile)) {
      return resolve(tempFile);
    }

    const filePath = path.resolve(__dirname, `../../@salesforce-ux/design-system/assets/icons/${category}/${name}.svg`);
    fs.readFile(filePath, function(err, data) {
      if (err) {
        reject(err);
      } else {
        const doc = new SVGDoc(data);
        doc.$svg.attr('id', `svg__${category}__${name}`);
        const content = doc.toString(SVGDoc.OUTPUT_FORMAT.SYMBOL).replace(' fill="#fff"', '');
        const svgModule = [
          'module.exports = \'',
          content,
          '\';\n'
        ];

        fs.writeFile(tempFile, svgModule.join(''), function(err1) {
          if (err1) {
            reject(err1);
          } else {
            resolve(tempFile);
          }
        });
      }
    });
  });

  return promise;
};

if (!fs.existsSync(TEMP_FOLDER)) {
  fs.mkdirSync(TEMP_FOLDER);
  categories.forEach((category) => {
    fs.mkdirSync(path.resolve(TEMP_FOLDER, category));
    const categoryFolder = path.resolve(SALESFORCE_UX_ICONS_ROOT, category);
    fs.readdir(categoryFolder, function(err, files) {
      files.forEach((file) => {
        if (file.endsWith('.svg')) {
          ensureSvg(category, file.substring(0, file.length - 4));
        }
      });
    });
  });
}

const process = function(source) {
  const promise = new Promise(function(resolve, reject) {
    if (source.indexOf('.embed(') > -1) {
      const reg = /.embed\((.|\n)*?\)/g;
      const matches = source.match(reg);
      const icons = [];

      matches.forEach((match) => {
        const iconString = match.replace(/\s|\[|\]/g, '').replace('.embed(', '').replace(')', '');
        if (iconString.startsWith('\'')) {
          icons.push(...iconString.split(','));
        }
      });

      const scripts = [];
      if (icons.length > 0) {
        scripts.push('\nvar __vue_lightning_sprite__ = require(\'svg-sprite-loader/lib/web/global-sprite.js\');');
        const arr = [];

        icons.forEach((icon) => {
          if (icon) {
            const names = icon.replace(/'/g, '').split('.');
            const category = names[0];
            const name = names[1];
            const tempFile = path.join(TEMP_FOLDER, category, `${name}.js`);
            scripts.push(`\n__vue_lightning_sprite__.add(require('${tempFile}'), 'symbols');`);
            arr.push(ensureSvg(category, name));
          }
        });

        Promise.all(arr).then(function() {
          resolve(scripts.join(''));
        }).catch(reject);
      } else {
        resolve('');
      }
    } else {
      resolve('');
    }
  });

  return promise;
};

module.exports = function(source) {
  const ext = path.extname(this.resourcePath).toLowerCase();
  let content = '';
  if (ext === '.vue') {
    content = fs.readFileSync(this.resourcePath).toString();
  } else if (ext === '.js') {
    content = source;
  }

  const callback = this.async();
  process(content).then(function(script) {
    if (script) {
      callback(null, `${script}\n\n${source}`);
    } else {
      callback(null, source);
    }
  }).catch(callback);
};
